﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using IronIntel.Services;
using IronIntel.Services.Users;
using IronIntel.Site;
using IronIntel.Contractor.Users;
using IronIntel.Services.Customers;
using System.Web;
using Foresight.Fleet.Services;
using Foresight.Fleet.Services.User;

namespace IronIntel.Contractor.Site
{
    public class ContractorBasePage : IronIntelBasePage
    {
        public static string AppVersion
        {
            get
            {
                return SystemParams.AppVersion;
            }
        }

        new public string PageTitle
        {
            get
            {
                if (MainStyleObj != null && !string.IsNullOrWhiteSpace(MainStyleObj.PageTitle))
                    return MainStyleObj.PageTitle;
                return base.PageTitle;
            }
        }

        private MainStyle _MainStyleObj;
        protected MainStyle MainStyleObj
        {
            get
            {
                if (_MainStyleObj == null)
                {
                    _MainStyleObj = SystemParams.GetMainStyle();
                }
                return _MainStyleObj;
            }
        }

        protected IronIntel.Contractor.Users.UserInfo GetCurrentUser()
        {
            var session = GetCurrentLoginSession();
            if (session == null)
            {
                return null;
            }
            return UserManagement.GetUserByIID(session.User.UID);
        }

        protected bool IsAdminOrSuper
        {
            get
            {
                var user = GetCurrentUser();
                if (user == null)
                {
                    return false;
                }
                return (user.UserType == Users.UserTypes.Admin || user.UserType == Users.UserTypes.SupperAdmin);
            }
        }

        public override string GetIronSystemServiceAddress()
        {
            return SystemParams.SystemServiceAddresses[0];
        }

        protected virtual bool AllowCurrentLoginSessionEnter()
        {
            var session = GetCurrentLoginSession();
            if (session == null)
            {
                return false;
            }
            if (string.Compare(session.User.CompanyID, SystemParams.CompanyID, true) == 0)
            {
                return true;
            }
            if (string.Compare(session.User.CompanyID, CompanyInfo.FORESIGHT, true) == 0)
            {
                return true;
            }
            return CreateClient<Foresight.Fleet.Services.User.UserQueryClient>().CanEnterSite(session.SessionID, SystemParams.CompanyID);
        }

        protected virtual bool ThrowIfNotAllowed { get { return false; } }

        protected virtual bool CanDirectAccess { get { return false; } }

        protected bool CheckUserToken()
        {
            var session = GetCurrentLoginSession();
            if (session != null)//已经登录
            {
                return true;
            }
            string tkstring = Request.Params["tk"];
            if (string.IsNullOrEmpty(tkstring))
                return false;

            try
            {
                var sc = new FI.FIC.Models.Security.SymmetricCrypt(FI.FIC.Models.Security.CryptType.DES);
                tkstring = sc.Decrypt(tkstring,
                    FI.FIC.DataProviders.ChartDataProvider.DES_Key,
  FI.FIC.DataProviders.ChartDataProvider.DES_IV);

                string[] temps = tkstring.Split('|');
                if (temps.Length != 2)
                    return false;

                string timestring = temps[0];
                DateTime time = DateTime.MinValue;
                if (!DateTime.TryParse(timestring, out time)
                    || time < DateTime.UtcNow.AddMinutes(-5))
                    return false;

                var sessionid = temps[1];
                var c = CreateClient<UserQueryClient>();
                //通过手机SessionID获取新的Web Session
                var newsession = c.GetNewLoginSession(sessionid, APPNAME);
                SetLoginSessionCookie(newsession.SessionID);
            }
            catch (Exception ex)
            {
                return false;
            }

            return true;
        }

        protected bool CheckLoginSession()
        {
            var session = GetCurrentLoginSession();
            if (session == null)
            {
                if (CanDirectAccess)
                    RedirectToLoginPageWithUrl();
                else
                    RedirectToLoginPage();
                return false;
            }
            if (!AllowCurrentLoginSessionEnter())
            {
                if (ThrowIfNotAllowed)
                {
                    throw new Exception("The user was not allowed to enter this page.");
                }
                else
                {
                    string entry = GetUserDefaultEntryPageUrl(session.User);
                    if (string.IsNullOrEmpty(entry))
                        Response.Redirect(entry, true);
                    else
                        Response.Redirect(LoginPageUrl, true);
                }
                return false;
            }
            return true;
        }

        protected void RedirectToLoginPageWithUrl()
        {
            string url = Request.Url.ToString();
            url = HttpUtility.UrlEncode(url);
            Response.Redirect(LoginPageUrl + "?f=" + url);
        }

        protected void DoLogout()
        {
            string sid = null;
            try
            {
                var session = GetCurrentLoginSession();
                if (session != null)
                {
                    sid = session.SessionID;
                }
            }
            catch { }
            try
            {
                ClearLoginSessionCookie();
            }
            catch { }
            if (sid != null)
            {
                try
                {
                    CreateClient<Foresight.Fleet.Services.User.UserQueryClient>().LogoutWithSessionID(sid);
                }
                catch
                {
                    // nothing
                }
            }

            RedirectToLoginPage();
        }

        protected void AddLog(string type, string source, string message, string detail)
        {
            try
            {
                SystemParams.WriteLog(type, source, message, detail);
            }
            catch
            {
                // nothing
            }
        }

        protected string GenerateUrl(string file)
        {
            string url;
            System.Web.UI.Page page = System.Web.HttpContext.Current.Handler as System.Web.UI.Page;
            if (page != null)
            {
                // Use page instance.
                url = page.ResolveUrl("~/") + file;
            }
            else
            {
                // avoid duplicate operation
                url = System.Web.HttpContext.Current.Request.ApplicationPath + "/" + file;
            }
            try
            {
                var path = System.IO.Path.Combine(System.Web.HttpContext.Current.Request.PhysicalApplicationPath, file);
                if (System.IO.File.Exists(path))
                {
                    url += "?t=" + System.IO.File.GetLastWriteTimeUtc(path).Ticks;
                }
            }
            catch (Exception)
            {
                // cant read file
            }
            return url;
        }

        protected byte[] ConvertFile2bytes(HttpPostedFile uploadFile)
        {
            byte[] dataBuffer = new byte[uploadFile.InputStream.Length];
            uploadFile.InputStream.Position = 0;
            uploadFile.InputStream.Read(dataBuffer, 0, dataBuffer.Length);
            uploadFile.InputStream.Close();
            return dataBuffer;
        }

        public virtual string JQueryVersion
        {
            get { return "1.8.0"; }
        }

        protected T CreateClient<T>(string companyid = null) where T : RemoteClientBase
        {
            var session = GetCurrentLoginSession();
            return FleetServiceClientHelper.CreateClient<T>(companyid, session == null ? "" : session.SessionID);
        }

        protected bool CheckRight(string custid, int featureid)
        {
            var user = GetCurrentUser();
            if (user == null)
                return false;

            if (user.UserType == Users.UserTypes.SupperAdmin)
                return true;

            if (user.UserType == Users.UserTypes.Common || user.UserType == Users.UserTypes.Admin)
            {
                var client = FleetServiceClientHelper.CreateClient<PermissionProvider>();
                Tuple<Feature, Permissions>[] pmss = client.GetUserPermissions(custid, user.IID);
                if (pmss.Length > 0)
                {
                    Tuple<Feature, Permissions> permission = pmss.FirstOrDefault(m => m.Item1.Id == featureid);
                    if (permission != null)
                        return true;
                }
            }
            return false;
        }
    }
}
